package api

import (
	"database/sql"
	"fmt"
	"log"
	"net/http"
	"time"

	"git.simponic.xyz/simponic/whois/api/template"
	"git.simponic.xyz/simponic/whois/api/types"
	"git.simponic.xyz/simponic/whois/api/whois"
	"git.simponic.xyz/simponic/whois/args"
	"git.simponic.xyz/simponic/whois/utils"
)

func LogRequestContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		context.Start = time.Now()
		context.Id = utils.RandomId()

		log.Println(req.Method, req.URL.Path, req.RemoteAddr, context.Id)
		return success(context, req, resp)
	}
}

func LogExecutionTimeContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		end := time.Now()
		log.Println(context.Id, "took", end.Sub(context.Start))

		return success(context, req, resp)
	}
}

func HealthCheckContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		resp.WriteHeader(200)
		resp.Write([]byte("healthy"))
		return success(context, req, resp)
	}
}

func FailurePassingContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(_success types.Continuation, failure types.Continuation) types.ContinuationChain {
		return failure(context, req, resp)
	}
}

func IdContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		return success(context, req, resp)
	}
}

func SetJsonContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		resp.Header().Set("Content-Type", "application/json")
		return success(context, req, resp)
	}
}

func SetCorsContinuation(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
	return func(success types.Continuation, _failure types.Continuation) types.ContinuationChain {
		resp.Header().Set("Access-Control-Allow-Origin", "*")
		return success(context, req, resp)
	}
}

func CacheControlMiddleware(next http.Handler, maxAge int) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		header := fmt.Sprintf("public, max-age=%d", maxAge)
		w.Header().Set("Cache-Control", header)
		next.ServeHTTP(w, r)
	})
}

// basically b(a(message)) if and only if b is successful
func Compose(a types.Continuation, b types.Continuation) types.Continuation {
	return func(context *types.RequestContext, req *http.Request, resp http.ResponseWriter) types.ContinuationChain {
		return func(success types.Continuation, failure types.Continuation) types.ContinuationChain {
			return b(context, req, resp)(a, FailurePassingContinuation)(success, failure)
		}
	}
}

func MakeMux(argv *args.Arguments, dbConn *sql.DB) *http.ServeMux {
	mux := http.NewServeMux()

	staticFileServer := http.FileServer(http.Dir(argv.StaticPath))
	mux.Handle("GET /static/", http.StripPrefix("/static/", CacheControlMiddleware(staticFileServer, 3600)))

	makeRequestContext := func() *types.RequestContext {
		return &types.RequestContext{
			DBConn:       dbConn,
			Args:         argv,
			TemplateData: &map[string]interface{}{},
		}
	}

	mux.HandleFunc("GET /health", func(w http.ResponseWriter, r *http.Request) {
		requestContext := makeRequestContext()
		LogRequestContinuation(requestContext, r, w)(HealthCheckContinuation, FailurePassingContinuation)(LogExecutionTimeContinuation, LogExecutionTimeContinuation)(IdContinuation, IdContinuation)
	})

	mux.HandleFunc("GET /", func(w http.ResponseWriter, r *http.Request) {
		requestContext := makeRequestContext()

		LogRequestContinuation(requestContext, r, w)(whois.FetchLatestName, FailurePassingContinuation)(template.TemplateContinuation("home.html", true), FailurePassingContinuation)(LogExecutionTimeContinuation, LogExecutionTimeContinuation)(IdContinuation, IdContinuation)
	})

	mux.HandleFunc("GET /updates", func(w http.ResponseWriter, r *http.Request) {
		requestContext := makeRequestContext()
    continuation := Compose(template.TemplateContinuation("updates.json.tmpl", false), Compose(whois.ListUpdates, Compose(SetJsonContinuation, SetCorsContinuation)))
		LogRequestContinuation(requestContext, r, w)(continuation, FailurePassingContinuation)(LogExecutionTimeContinuation, LogExecutionTimeContinuation)(IdContinuation, IdContinuation)
	})

	return mux
}
